@@ -1030,6 +1030,7 @@ gssdp_client_clear_headers (GSSDPClient *client)
 
         g_list_free_full (client->priv->headers,
                           (GDestroyNotify) header_field_free);
+        client->priv->headers = NULL;
 }
 
 /**
@@ -1125,7 +1126,7 @@ make_server_id (void)
         
         return g_strdup_printf ("%s/%s GSSDP/%s",
                                 sysinfo.sysname,
-                                sysinfo.version,
+                                sysinfo.release,
                                 VERSION);
 #endif
 }
@@ -1227,6 +1228,7 @@ socket_source_cb (GSSDPSocketSource *socket_source, GSSDPClient *client)
         GInputVector vector;
         GSocketControlMessage **messages;
         gint num_messages;
+        gboolean ret = TRUE;
 
         vector.buffer = buf;
         vector.size = BUF_SIZE;
@@ -1244,8 +1246,13 @@ socket_source_cb (GSSDPSocketSource *socket_source, GSSDPClient *client)
                                           &error);
 
         if (bytes == -1) {
-                g_warning ("Failed to receive from socket: %s",
-                           error->message);
+                if (!g_error_matches (error,
+                                      G_IO_ERROR,
+                                      G_IO_ERROR_WOULD_BLOCK)) {
+                        g_warning ("Failed to receive from socket: %s",
+                                   error->message);
+                        ret = FALSE;
+                }
 
                 goto out;
         }
@@ -1392,7 +1399,7 @@ out:
                 g_free (messages);
         }
 
-        return TRUE;
+        return ret;
 }
 
 static gboolean
@@ -1400,11 +1407,34 @@ request_socket_source_cb (G_GNUC_UNUSED GIOChannel  *source,
                           G_GNUC_UNUSED GIOCondition condition,
                           gpointer                   user_data)
 {
-        GSSDPClient *client;
+        GSSDPClient *client = GSSDP_CLIENT (user_data);
+        GSSDPSocketSource *request_socket = NULL;
+        GError *error = NULL;
 
-        client = GSSDP_CLIENT (user_data);
+        if (socket_source_cb (client->priv->request_socket, client))
+                return TRUE;
 
-        return socket_source_cb (client->priv->request_socket, client);
+        request_socket = gssdp_socket_source_new (
+                                        GSSDP_SOCKET_SOURCE_TYPE_REQUEST,
+                                        gssdp_client_get_host_ip (client),
+                                        client->priv->socket_ttl,
+                                        gssdp_client_get_interface (client),
+                                        &error);
+        if (request_socket != NULL) {
+                g_clear_object (&client->priv->request_socket);
+                client->priv->request_socket = request_socket;
+                gssdp_socket_source_set_callback (
+                                        client->priv->request_socket,
+                                        (GSourceFunc) request_socket_source_cb,
+                                        client);
+                gssdp_socket_source_attach (client->priv->request_socket);
+        } else {
+                g_warning ("Could not recreate request socket on error: %s",
+                           error->message);
+                g_clear_error (&error);
+        }
+
+        return TRUE;
 }
 
 static gboolean
@@ -1412,11 +1442,34 @@ multicast_socket_source_cb (G_GNUC_UNUSED GIOChannel  *source,
                             G_GNUC_UNUSED GIOCondition condition,
                             gpointer                   user_data)
 {
-        GSSDPClient *client;
+        GSSDPClient *client = GSSDP_CLIENT (user_data);
+        GSSDPSocketSource *multicast_socket = NULL;
+        GError *error = NULL;
 
-        client = GSSDP_CLIENT (user_data);
+        if (socket_source_cb (client->priv->multicast_socket, client))
+                return TRUE;
 
-        return socket_source_cb (client->priv->multicast_socket, client);
+        multicast_socket = gssdp_socket_source_new (
+                                        GSSDP_SOCKET_SOURCE_TYPE_REQUEST,
+                                        gssdp_client_get_host_ip (client),
+                                        client->priv->socket_ttl,
+                                        gssdp_client_get_interface (client),
+                                        &error);
+        if (multicast_socket != NULL) {
+                g_clear_object (&client->priv->multicast_socket);
+                client->priv->multicast_socket = multicast_socket;
+                gssdp_socket_source_set_callback (
+                                        client->priv->multicast_socket,
+                                        (GSourceFunc)multicast_socket_source_cb,
+                                        client);
+                gssdp_socket_source_attach (client->priv->multicast_socket);
+        } else {
+                g_warning ("Could not recreate search socket on error: %s",
+                           error->message);
+                g_clear_error (&error);
+        }
+
+        return TRUE;
 }
 
 static gboolean
@@ -1424,8 +1477,34 @@ search_socket_source_cb (G_GNUC_UNUSED GIOChannel  *source,
                          G_GNUC_UNUSED GIOCondition condition,
                          gpointer                   user_data)
 {
-        GSSDPClient *client;
+        GSSDPClient *client = GSSDP_CLIENT (user_data);
+        GSSDPSocketSource *search_socket = NULL;
+        GError *error = NULL;
 
+        if (socket_source_cb (client->priv->search_socket, client))
+                return TRUE;
+
+        search_socket = gssdp_socket_source_new (
+                                        GSSDP_SOCKET_SOURCE_TYPE_REQUEST,
+                                        gssdp_client_get_host_ip (client),
+                                        client->priv->socket_ttl,
+                                        gssdp_client_get_interface (client),
+                                        &error);
+        if (search_socket != NULL) {
+                g_clear_object (&client->priv->search_socket);
+                client->priv->search_socket = search_socket;
+                gssdp_socket_source_set_callback (
+                                        client->priv->search_socket,
+                                        (GSourceFunc)search_socket_source_cb,
+                                        client);
+                gssdp_socket_source_attach (client->priv->search_socket);
+        } else {
+                g_warning ("Could not recreate search socket on error: %s",
+                           error->message);
+                g_clear_error (&error);
+        }
+
+        return TRUE;
         client = GSSDP_CLIENT (user_data);
 
         return socket_source_cb (client->priv->search_socket, client);
